package sun.java2d.loops;

import sun.java2d.pipe.Region;
import sun.awt.image.BufImgSurfaceData;
import java.awt.image.BufferedImage;
import java.awt.Composite;
import sun.java2d.SurfaceData;
import sun.java2d.SunGraphics2D;

public class MaskFill extends GraphicsPrimitive
{
    public static final String methodSignature;
    public static final String fillPgramSignature;
    public static final String drawPgramSignature;
    public static final int primTypeID;
    private static RenderCache fillcache;
    
    public static MaskFill locate(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        return (MaskFill)GraphicsPrimitiveMgr.locate(MaskFill.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    public static MaskFill locatePrim(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        return (MaskFill)GraphicsPrimitiveMgr.locatePrim(MaskFill.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    public static MaskFill getFromCache(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        Object value = MaskFill.fillcache.get(surfaceType, compositeType, surfaceType2);
        if (value != null) {
            return (MaskFill)value;
        }
        MaskFill locatePrim = locatePrim(surfaceType, compositeType, surfaceType2);
        if (locatePrim != null) {
            MaskFill.fillcache.put(surfaceType, compositeType, surfaceType2, locatePrim);
            return locatePrim;
        }
        return locatePrim;
    }
    
    protected MaskFill(String s, SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        super(s, MaskFill.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    protected MaskFill(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        super(MaskFill.methodSignature, MaskFill.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    public MaskFill(long n, SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        super(n, MaskFill.methodSignature, MaskFill.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    public native void MaskFill(SunGraphics2D p0, SurfaceData p1, Composite p2, int p3, int p4, int p5, int p6, byte[] p7, int p8, int p9);
    
    public native void FillAAPgram(SunGraphics2D p0, SurfaceData p1, Composite p2, double p3, double p4, double p5, double p6, double p7, double p8);
    
    public native void DrawAAPgram(SunGraphics2D p0, SurfaceData p1, Composite p2, double p3, double p4, double p5, double p6, double p7, double p8, double p9, double p10);
    
    public boolean canDoParallelograms() {
        return this.getNativePrim() != 0L;
    }
    
    @Override
    public GraphicsPrimitive makePrimitive(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        if (SurfaceType.OpaqueColor.equals(surfaceType) || SurfaceType.AnyColor.equals(surfaceType)) {
            if (CompositeType.Xor.equals(compositeType)) {
                throw new InternalError("Cannot construct MaskFill for XOR mode");
            }
            return new General(surfaceType, compositeType, surfaceType2);
        }
        else {
            throw new InternalError("MaskFill can only fill with colors");
        }
    }
    
    @Override
    public GraphicsPrimitive traceWrap() {
        return new MaskFill.TraceMaskFill(this);
    }
    
    static {
        methodSignature = "MaskFill(...)".toString();
        fillPgramSignature = "FillAAPgram(...)".toString();
        drawPgramSignature = "DrawAAPgram(...)".toString();
        primTypeID = GraphicsPrimitive.makePrimTypeID();
        MaskFill.fillcache = new RenderCache(10);
        GraphicsPrimitiveMgr.registerGeneral(new MaskFill(null, null, null));
    }
    
    private static class General extends MaskFill
    {
        FillRect fillop;
        MaskBlit maskop;
        
        public General(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
            super(surfaceType, compositeType, surfaceType2);
            this.fillop = FillRect.locate(surfaceType, CompositeType.SrcNoEa, SurfaceType.IntArgb);
            this.maskop = MaskBlit.locate(SurfaceType.IntArgb, compositeType, surfaceType2);
        }
        
        @Override
        public void MaskFill(SunGraphics2D sunGraphics2D, SurfaceData surfaceData, Composite composite, int n, int n2, int width, int height, byte[] array, int n3, int n4) {
            SurfaceData data = BufImgSurfaceData.createData(new BufferedImage(width, height, 2));
            Region clipRegion = sunGraphics2D.clipRegion;
            sunGraphics2D.clipRegion = null;
            int pixel = sunGraphics2D.pixel;
            sunGraphics2D.pixel = data.pixelFor(sunGraphics2D.getColor());
            this.fillop.FillRect(sunGraphics2D, data, 0, 0, width, height);
            sunGraphics2D.pixel = pixel;
            sunGraphics2D.clipRegion = clipRegion;
            this.maskop.MaskBlit(data, surfaceData, composite, null, 0, 0, n, n2, width, height, array, n3, n4);
        }
    }
    
    private static class TraceMaskFill extends MaskFill
    {
        MaskFill target;
        MaskFill fillPgramTarget;
        MaskFill drawPgramTarget;
        
        public TraceMaskFill(MaskFill target) {
            super(target.getSourceType(), target.getCompositeType(), target.getDestType());
            this.target = target;
            this.fillPgramTarget = new MaskFill(MaskFill.TraceMaskFill.fillPgramSignature, target.getSourceType(), target.getCompositeType(), target.getDestType());
            this.drawPgramTarget = new MaskFill(MaskFill.TraceMaskFill.drawPgramSignature, target.getSourceType(), target.getCompositeType(), target.getDestType());
        }
        
        @Override
        public GraphicsPrimitive traceWrap() {
            return this;
        }
        
        @Override
        public void MaskFill(SunGraphics2D sunGraphics2D, SurfaceData surfaceData, Composite composite, int n, int n2, int n3, int n4, byte[] array, int n5, int n6) {
            GraphicsPrimitive.tracePrimitive(this.target);
            this.target.MaskFill(sunGraphics2D, surfaceData, composite, n, n2, n3, n4, array, n5, n6);
        }
        
        @Override
        public void FillAAPgram(SunGraphics2D sunGraphics2D, SurfaceData surfaceData, Composite composite, double n, double n2, double n3, double n4, double n5, double n6) {
            GraphicsPrimitive.tracePrimitive(this.fillPgramTarget);
            this.target.FillAAPgram(sunGraphics2D, surfaceData, composite, n, n2, n3, n4, n5, n6);
        }
        
        @Override
        public void DrawAAPgram(SunGraphics2D sunGraphics2D, SurfaceData surfaceData, Composite composite, double n, double n2, double n3, double n4, double n5, double n6, double n7, double n8) {
            GraphicsPrimitive.tracePrimitive(this.drawPgramTarget);
            this.target.DrawAAPgram(sunGraphics2D, surfaceData, composite, n, n2, n3, n4, n5, n6, n7, n8);
        }
        
        @Override
        public boolean canDoParallelograms() {
            return this.target.canDoParallelograms();
        }
    }
}
